#!/bin/zsh

# Copyright 2014 - 2016 Paul Hänsch
#
# This file is part of Confetti.
# 
# Confetti is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Confetti is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Confetti.  If not, see <http://www.gnu.org/licenses/>. 

BR='
'

case $PROFILE in
medical)
  view_card="$_EXEC/templates/view_client.sh"
  edit_card="$_EXEC/templates/edit_client.sh"
  _GET[order]="${_GET[order]:-lastname}"
  _GET[filtertype]="${_GET[filtertype]:-name}"
  profile_medical=x
;;
circus)
  view_card="$_EXEC/templates/view_attendee.sh"
  edit_card="$_EXEC/templates/edit_attendee.sh"
  _GET[order]="${_GET[order]:-firstname}"
  _GET[filtertype]="${_GET[filtertype]:-any}"
  profile_circus=x
;;
esac

edit="${_GET[edit]}"
[ \! -f "vcard/$edit" -a \! -f "temp/$edit" ] && edit=''

listcourses() {
  ls -1 ${_DATA}/ical/*ics |while read file; do
    icstime="$(sed -rn 's:^DTSTART\:(TZID=.*\:)?([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})Z?\r$:\2-\3-\4 \5\:\6\:\7:p' "$file")"
    echo "$(date -d "$icstime" "+%u %H%M%S")\t$file"
  done |sort |sed -r 's:^.*\t(.*/)([^/]+)$:\2:'
}

list_hi_companies(){
  sed -rn 's;^X-HEALTH-INSURANCE:([^\;]+)\;.*$;\1;p' ${_DATA}/vcard/*vcf \
  | sort -u
}

listcards() {
  case "${_GET[filtertype]}" in
    any)
       grep -il "${_GET[filter]}" ${_DATA}/vcard/*vcf
      ;;
    name)
       egrep -xil "(FN|NICKNAME|N)(;.+)*:.*${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    street)
       egrep -xil "(ADR)(;.+)*:([^;]*;){2}${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    zip)
       egrep -xil "(ADR)(;.+)*:([^;]*;){5}${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    telephone)
       egrep -xil "(TEL)(;.+)*:.*${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    birth)
       egrep -xil "(BDAY)(;.+)*:${_GET[filter]}.*" ${_DATA}/vcard/*vcf
      ;;
    course)
      ;;
    *) ls -1 ${_DATA}/vcard/*vcf 2>/dev/null
      ;;
  esac |case "${_GET[order]}" in
    firstname)
      while read file; do
        fn=$(sed -rn 's:^N(;.+)*\:([^;]*;){1} *([^;]*).*$:\3:p' "$file")
	echo "$fn\t$file"
      done
      ;;
    lastname)
      while read file; do
        ln=$(sed -rn 's:^N(;.+)*\:([^;]*;){0} *([^;]*).*$:\3:p' "$file")
	echo "$ln\t$file"
      done
      ;;
    bdate)
      while read file; do
        bd=$(sed -rn 's:^BDAY(;.+)*\:(.*)$:\2:p' "$file")
	echo "$bd\t$file"
      done
      ;;
    *)
      sed -r 's:^.*$:x\t&:'
      ;;
  esac |sort |sed -r 's:^.*\t(.*/)([^/]+)$:\2:'
}

vcf_parse() {
  tr -d '\n' <"$1" |sed -r 's:\r ::g;s:\r:\n:g' \
  | sed -rn '
    s:^X-MS-CARDPICTURE:PHOTO:p;
    s:^X-GENDER:GENDER:p;
    s:^X-ANNIVERSARY:ANNIVERSARY:p;
    s:^X-EVOLUTION-ANNIVERSARY:ANNIVERSARY:p;
    s:^X-KADDRESSBOOK-X-Anniversary:ANNIVERSARY:p;
    s:^X-AIM(;[^"\:]+|;"[^"]+")*\:(.*)$:IMPP\1\:aim\:\2:p;
    s:^X-ICQ(;[^"\:]+|;"[^"]+")*\:(.*)$:IMPP\1\:aim\:\2:p;
    s:^X-GOOGLE-TALK(;[^"\:]+|;"[^"]+")*\:(.*)$:IMPP\1\:xmpp\:\2:p;
    s:^X-JABBER(;[^"\:]+|;"[^"]+")*\:(.*)$:IMPP\1\:xmpp\:\2:p;
    s:^X-MSN(;[^"\:]+|;"[^"]+")*\:(.*)$:IMPP\1\:msn\:\2:p;
    s:^X-YAHOO(;[^"\:]+|;"[^"]+")*\:(.*)$:IMPP\1\:ymsgr\:\2:p;
    s:^X-SIP(;[^"\:]+|;"[^"]+")*\:(sip\:)?(.*)$:IMPP\1\:sip\:\3:p;
    s:^LABEL(;[^"\:]+|;"[^"]+")*\:(.*)$:ADR;LABEL="\2"\1\::p;
    s:^X-EVOLUTION-BLOG-URL:URL:p;

    s:^AGENT:RELATED\;TYPE=agent:p;
    s:^X-ASSISTANT:RELATED\;TYPE=assistant;VALUE=text:p;
    s:^X-EVOLUTION-ASSISTANT:RELATED\;TYPE=assistant;VALUE=text:p;
    s:^X-KADDRESSBOOK-X-AssistantsName:RELATED\;TYPE=assistant;VALUE=text:p;
    s:^X-MANAGER:RELATED\;TYPE=manager;VALUE=text:p;
    s:^X-EVOLUTION-MANAGER:RELATED\;TYPE=manager;VALUE=text:p;
    s:^X-KADDRESSBOOK-X-ManagersName:RELATED\;TYPE=manager;VALUE=text:p;
    s:^X-SPOUSE:RELATED\;TYPE=spouse;VALUE=text:p;
    s:^X-EVOLUTION-SPOUSE:RELATED\;TYPE=spouse;VALUE=text:p;
    s:^X-KADDRESSBOOK-X-SpouseName:RELATED\;TYPE=spouse;VALUE=text:p;

    s:^([A-Z].*)$:\1:p;
    ' \
  | sed -r 's:^([^;\:]+)(;[^"\:]+|;"[^"]+")*\:(.*)$:key="\1"\nvalue="\3"\ntag=\2:g' \
  | while read -r line; do
    case "$line" in
      key=*) printf %s\\n "$line"
	;;
      value=*) printf %s\\n "${line}"
	;;
      tag=*) ot=''
	     printf %s "$line" \
	     | sed -r 's:^tag=::;s:\;([A-Z+_-]+="[^"]+"|[A-Z+_-]+=[^\;]+):\n\1:g;' \
	     | sed -r 's:([A-Z+_-]+)="?(.*)"?:tag\[\1\]="\2":g' \
	     | sed -r '/^ *$/d' \
	     | sort |while read -r tag; do
	       nt="$(printf %s "$tag" |sed -r 's:^tag\[([A-Z+_-]+)\]="(.*)"$:\1:')"
	       nv="$(printf %s "$tag" |sed -r 's:^tag\[([A-Z+_-]+)\]="(.*)"$:\2:')"
	       [ "$nt" = "$ot" ] && vl="$nv,$vl" || vl="$nv"
	       printf %s\\n "tag[$nt]=\"$vl\""
	       ot="$nt"
	     done
	;;
    esac
  done \
  | sed -r 's:[\\$`]:\\&:g'
}

view_card() {  #Parameter: Cardfile
  id="$1"
  cardfile="$_DATA/vcard/${id}"
  cachefile="$_DATA/cache/${id}.cache"
  unset key
  if [ "$cachefile" -nt "$cardfile" ]; then
    cat "$cachefile"
  else
    declare -A tags
    declare -A values
    vcf_parse "$cardfile" |while read -r line; do
      declare -A tag
      case "$line" in
        value*) eval "$line";;
        tag*)   eval "$line";;
        key*)
          if [ -z "$key" ]; then
            eval "$line"
          else
            values[$key]="${value//\\r\\n/$BR}"
            for t in ${(k)tag}; do
              tags[${key}_$t]="$tag[$t]"
            done
            eval "$line"
            if [ -n "$values[$key]" ]; then
              n=0
              while [ -n "$values[$key$n]" ]; do n=$(($n + 1)); done
              key=$key$n
            fi
            unset value
            unset tag
          fi
        ;;
      esac
    done
    . $view_card |tee "$cachefile"
  fi
}

edit_card() {  #Parameter: Cardfile
  id="$1"
  cardfile="$_DATA/vcard/$id"
  tempfile="$_DATA/temp/$id"
  [ -f "$tempfile" ] && cardfile="$tempfile"
  unset key

  declare -A tags
  declare -A values
  vcf_parse "$cardfile" |while read -r line; do
    declare -A tag
    case "$line" in
      value*) eval "$line";;
      tag*)   eval "$line";;
      key*)
        if [ -z "$key" ]; then
          eval "$line"
        else
          [ -n "$value" ] && values[$key]="${value//\\r\\n/$BR}" || values[$key]='\r'
          for t in ${(k)tag}; do
            tags[${key}_$t]="$tag[$t]"
          done
          eval "$line"
          if [ -n "$values[$key]" ]; then
            n=0
            while [ -n "$values[${key}${n}]" ]; do n=$(($n + 1)); done
            key=$key$n
          fi
            unset value
            unset tag
        fi
      ;;
    esac
  done
  . $edit_card
}
